from scipy.integrate import trapz, cumtrapz
import numpy as np
from scipy.integrate import dblquad

def poisson_with_prior(R, VT, Nobs, prior=lambda x: 1):

    if callable(prior):
        likelihood = R**Nobs * np.exp(-R * VT)
        norm = trapz(likelihood * prior(R), R)
        return likelihood * prior(R) / norm
        
def kkl_with_prior(vt1, vt2, R, prior = lambda x: 1):

    likelihood = (np.exp(-R * vt2) * (R * (vt1 - vt2) - 2 ) + np.exp(-R * vt1) * (R * (vt1 - vt2) + 2)) \
                 / (R * (vt1 - vt2) ** 3)

    norm = trapz(likelihood * prior(R), R)
    return likelihood * prior(R) / norm
def kkl_n_counts(vt1, vt2, R, n1 = 1, n2 = 1, prior = lambda x: 1, Nf = 500):
    f = np.linspace(0,1,Nf)
    RR, ff = np.meshgrid(R, f, indexing='ij')
    
    integrand = RR**(n1+n2) * ff**n1 * (1-ff)**n2 * np.exp(-RR*vt1*ff - RR*vt2*(1-ff))
    likelihood = trapz(integrand,f)
       
    norm = trapz(likelihood * prior(R), R)
    return likelihood * prior(R) / norm


def integrand_3class(f1, f2, r, vt1, vt2, vt3):
    return r**3 * f1 * f2 * (1.0-f1-f2) * np.exp(-r*(f1*vt1 + f2*vt2 + (1-f1-f2)*vt3))

def kkl_3class(vt1, vt2, vt3, R, prior = lambda x: 1, Nf = 500):
    likelihood = []
    
    for r in R:
        integral = dblquad(integrand_3class, 0, 1, lambda x: 0, lambda x:1-x, args=(r, vt1,vt2,vt3))
        likelihood.append(integral[0])
    
    likelihood = np.array(likelihood)
    norm = trapz(likelihood * prior(R), R)
    return likelihood * prior(R) / norm

def get_CI(Ps,R,CL):
    CDF = cumtrapz(Ps, R)
    i_span = np.where( (CDF>0.5-CL/2) & (CDF<0.5+CL/2) )[0]
    ilow, ihigh = i_span[0], i_span[-1]
    imed = np.where(CDF>0.5)[0][0]
    rmed = R[imed]
    rmax = R[np.argmax(Ps)]
    print('rmax: ',rmax)
    print(R[ilow], rmed, R[ihigh], CDF[-1])
    rplus, rminus = R[ihigh] - rmed, rmed - R[ilow]
    return rminus, rmed, rplus

def get_bounds_single(R, VT, Nobs, CL, fp, prior= lambda x: 1):
    Ps = fp(R, VT, Nobs, prior)
    CDF = cumtrapz(Ps, R)
    i_span = np.where( (CDF>0.5-CL/2) & (CDF<0.5+CL/2) )[0]
    ilow, ihigh = i_span[0], i_span[-1]
    imed = np.where(CDF>0.5)[0][0]
    rmed = R[imed]
    rmax = R[np.argmax(Ps)]
    print('rmax: ',rmax)
    print(R[ilow], rmed, R[ihigh], CDF[-1])
    rplus, rminus = R[ihigh] - rmed, rmed - R[ilow]
    return rminus, rmed, rplus
def get_bounds_kkl(R, vt2, vt1, CL, prior=lambda x: 1):
    Ps = kkl_with_prior(vt1, vt2, R, prior)
    CDF = cumtrapz(Ps, R)
    i_span = np.where( (CDF>0.5-CL/2) & (CDF<0.5+CL/2) )[0]
    ilow, ihigh = i_span[0], i_span[-1]
    imed = np.where(CDF>0.5)[0][0]
    rmed = R[imed]
    rmax = R[np.argmax(Ps)]
    print('rmax  ',rmax)
    print(R[ilow], rmed, R[ihigh], CDF[-1])
    rplus, rminus = R[ihigh] - rmed, rmed - R[ilow]
    return rminus, rmed, rplus
